name = 'justengel_theme'
version = '0.0.3'
description = 'Jinja theme capability'
url = 'https://github.com/justengel-web/justengel_theme'
author = 'Justin Engel'
author_email = 'jtengel08@gmail.com'
